#!/usr/bin/perl

# RogBlog 0.1
# by Roger Schlafly
# http://www.schlafly.net/blog/rogblog.htm

use strict;
require 5.004;
use Net::FTP;			# for uploading
use LWP::Simple;		# just use get() to retrieve page
use URI::Escape;

# Suggest: rogblog.pl -g -a ftp.server.com:/userdirectory:username:password

## Users can edit this data.
my $max = 30;		# of posts on front page
my $csv = "posts.csv";
my $frame = "frame";
my @allposts = ("posts1.htm", "posts2.htm", "posts3.htm", "posts4.htm", "posts5.htm");

## Code below does not need to be changed.

# Process the command line
if (!@ARGV)
{
	print "Usage: rogblog.pl [-g] [-a <account>] [-all]\n";
	exit 0;
}
my $generate;
my $all;
my @acct;
while (@ARGV)
{
	my $i = shift;
	if ($i eq "-g") { $generate = 1;}
	elsif ($i eq "-all") { $all = 1;}
	elsif ($i eq "-a") { @acct = split ":", shift;}
}

# Use a simple database for index and date/time for each post.

my ($i,$src,$x);
my %posts;
my %dates = &hashload($csv);
my $poster;
my @files;

# Scan the posts, correlating them with the date/time database.
foreach $src (@allposts)
{
	local *F;
	open(F,"<$src") or die("open file: $!");
	$i = 0;
	while (<F>)
	{
		if ($_ =~ /<hr/)
		{
			++$i;
			my $ind = "$src-$i";
			&hashadd($csv,$ind,$dates{$ind}=&get_date_time)
				unless (defined $dates{$ind});
			$posts{"$dates{$ind}+$i"} = $x;
			$x = "";
			$poster = $1 if (/blogger="(.+?)"/);
		}
		else
		{
			s/^(> )+//;		# for quoted blocks from email
			$x .= $_;
		}
	}
	close F;
}

# Combine the template, posts, and dates into html pages.
my @months = qw!Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec!;
my @days = qw!Sun Mon Tues Wednes Thurs Fri Satur!;
my (%pages,$j,$last,$k);

	foreach $j (sort { $b cmp $a } keys %posts)
	{
		my ($date,$time,$dweek,$i) = split quotemeta "+", $j;
		my @t = split "-", $date;
		my $datetitle = qq|$days[$dweek]day, $months[$t[1]-1] $t[2], $t[0]|;
		my $tag = "$t[0]-$t[1]";
		my $label = "$date+$time+$i";
		my $title = "";
		$title = $1 if $posts{$j} =~ s!<h3>(.*)</h3>!!s;
		my $x = "";
		$x .= qq|<br><p><div class="date">$datetitle</div>\n| if $last ne $date;
		$last = $date;
		$x .= qq|<div class="posts"><a name="$label">&nbsp;</a><br>
		<b>$title</b><br>
		$posts{$j}
		<br><span class="byline">posted by $poster at <a href="$tag.htm#$label">$time</a></span></div>
		|;
		$pages{$tag} .= $x;
		$pages{index} .= $x if ++$k <= $max;
		my $d = substr $days[$dweek], 0, 3;

		# Escape <, >, and &, and to produce valid RSS
		my (%escape1) = ('<'=>'&lt;', '>'=>'&gt;', '&'=>'&amp;', '"'=>'&quot;');
		my $escape_re  = join '|' => keys %escape1;
#		$title =~ s/($escape_re)/$escape1{$1}/g;
#		$posts{$j} =~ s/($escape_re)/$escape1{$1}/g;
		$posts{$j} =~ s!</?blockquote>!**!g;
		$posts{$j} =~ s!<p>!!g;
		$posts{$j} =~ s!</?i>!!g;
		$posts{$j} =~ s!</?b>!!g;
		$posts{$j} =~ s!</a>!!g;
		$posts{$j} =~ s!<a.+>!!g;
		$posts{$j} =~ s!<br>!!gi;
		$posts{$j} =~ s!<li>!!g;
		$posts{$j} =~ s!</?font.*?>!!gi;
		$posts{$j} =~ s!</?ul>!!g;
		$posts{$j} =~ s!<g>!(grin)!g;

		$pages{feed} .= qq|
	<item>
		<title>$title</title>
		<description>$posts{$j}</description>
		<link>myblogurl$tag.htm#$label</link>
		<pubDate>$d, $t[2] $months[$t[1]-1] $t[0] $time PST</pubDate>
	</item>
| if $k <= $max;
	}

# add the archive links into the frame
my $archivelinks;
foreach $j (sort keys %pages)
{
	my @t = split "-", $j;
	my $label = $j eq "index" ? "Current page" : "$months[$t[1]-1] $t[0]";
	$archivelinks .= qq|\t<a href="$j.htm">$label</a><br>\n| if $j ne "feed";
}

# drop the html pages into disk files
my ($urlinfo1,$urlinfo2);
foreach $j (keys %pages)
{
	local *F;
	my $suf = ($j eq "feed") ? "rss" : "htm";
	open(F,"<$frame.$suf") or die("open file: $!");
	my $y = "";
	while (<F>)
	{
		s/=====+/$pages{$j}/;
		s/archivelinks/$archivelinks/;
		$y .= $_;
		$urlinfo1 = $1 if (/blogtitle.*?>(.+?)</);
		$urlinfo2 = $1 if (/"(http.+?)".+blogurl/);
	}
	close F;
	$y =~ s/myblogurl/$urlinfo2/gs;		# single line
	# Write the html file.
	open(F,">$j.$suf") or die("open file: $!");
	print F $y;
	close F;
	push @files, "$j.$suf";
}

#@files = ("index.htm", "feed.rss") if !defined $all;

# Upload to the FTP site.
if (@acct)
{
	my $ftp = Net::FTP->new($acct[0], Debug => 0)
		or die "Could not connect: $@\n";
	$ftp->login($acct[2],$acct[3]);
	$ftp->binary();
	$ftp->cwd($acct[1]);
	my $i = 0;
	foreach $j (sort { $b cmp $a;} @files)
	{
		print "uploading $j\n";
		$ftp->put("$j","$j");
		last if ++$i > 2 and !defined $all;
	}
	$ftp->quit;
	if ($urlinfo1 and $urlinfo2)
	{
		# tell weblogs.com that i've updated
		my $page = get("http://newhome.weblogs.com/pingSiteForm?name="
			.&uri_escape($urlinfo1)."&url=".&uri_escape($urlinfo2));
		print "weblogs.com returned ", +split("\n",$page)+0," lines\n";
	}
}

exit 0;

sub get_date_time
{
	# Returns the date-time in the format "yyyy-mm-dd+hh:mm:ss+dweek".
	# Note: 12:08 AM shows up as 00:08:00
	my ($sec, $min, $hour, $day, $mon, $year, $dweek, $dyear, $daylight)
		= localtime(time());
	$year += 1900;
	$mon += 1;	# so in 1..12
	$mon < 10 and $mon = "0$mon";
	$day < 10 and $day = "0$day";
	$hour < 10 and $hour = "0$hour";
	$min < 10 and $min = "0$min";
	$sec < 10 and $sec = "0$sec";
	return "$year-$mon-$day+$hour:$min:$sec+$dweek";
}

sub hashadd
{
	# 1st arg = filename
	# appends remaining args to a line in file
	my @arg = @_;
	my $file = shift @arg;
	local *F;
	open(F,">>$file") or die("open file: $!");
	print F join(",",@arg),"\n";
	close(F);
}

sub hashload
{
	my ($file) = @_;
	local *F;
	open(F,"<$file") or die("open file: $!");
	my %all;
	while (<F>)
	{
		chomp;
		my @l = split ",",$_;
		$all{$l[0]} = $l[1];
	}
	close(F);
	return %all;
}

__END__

A simple program for maintaining a blog.
Goals:
Simple.
Use static web pages with plain vanilla html.
Write text with ordinary editor and html tags as needed.
Use a template for page layout.
Automatically track dates, merge html, and upload to server.
Easily test html offline.
Archive monthly.

Uses:
rogblog.pl - perl program to process and post
posts1.htm - source file for messages
posts.csv - generated for post dates
index.htm - generated
2003-01.htm - generated
